﻿using System;
using Messages;
using NServiceBus;
using NServiceBus.Logging;
using NServiceBus.Saga;

namespace EndpointWithSaga
{
    public class InventoryImageImportSaga : Saga<InventoryImageSagaData>, 
        IAmStartedByMessages<StartImportInventoryImagesMessage>,
        IHandleMessages<UploadImageMessage>,
        IHandleTimeouts<CancelImageUploadProcessMessage>
    {
        static ILog log = LogManager.GetLogger<InventoryImageImportSaga>();

        protected override void ConfigureHowToFindSaga(SagaPropertyMapper<InventoryImageSagaData> mapper)
        {
            mapper.ConfigureMapping<StartImportInventoryImagesMessage>(m => m.InventoryId).ToSaga(s => s.InventoryId);
            mapper.ConfigureMapping<UploadImageMessage>(m => m.InventoryId).ToSaga(s => s.InventoryId);
        }

        public void Handle(StartImportInventoryImagesMessage message)
        {
            log.Info($"Received StartImportInventoryImagesMessage message with InventoryId = '{message.InventoryId}' Description = {message.Description}");
            Data.InventoryId = message.InventoryId;
            Data.Description = message.Description;

            RequestTimeout<CancelImageUploadProcessMessage>(TimeSpan.FromSeconds(10));
            Bus.Send(new UploadImageMessage {InventoryId = Data.InventoryId});
        }

        public void Handle(UploadImageMessage message)
        {
            log.Info($"Received UploadImageMessage message with InventoryId = '{message.InventoryId}'");
        }

        public void Timeout(CancelImageUploadProcessMessage state)
        {
            log.Info($"Received CancelImageUploadProcessMessage message for InventoryId = '{Data.InventoryId}'");
            log.Info("Marking as completed");
            MarkAsComplete();
        }
    }

    public class InventoryImageSagaData : ContainSagaData
    {
        [Unique]
        public string InventoryId { get; set; }
        public string Description { get; set; }
    }

    public class CancelImageUploadProcessMessage
    {
        
    }
}