﻿using System;
using System.Reflection;
using Messages;
using NServiceBus;
using NServiceBus.Config;
using NServiceBus.Config.ConfigurationSource;
using NServiceBus.Persistence;
using NServiceBus.SagaPersisters;

namespace EndpointWithSaga
{
    class Program
    {
        static void Main(string[] args)
        {
            Console.Title = "EndpointWithSaga";
            var busConfiguration = new BusConfiguration();

            const string connectionString = "UseDevelopmentStorage=true";

            busConfiguration.EndpointName("EndpointWithSaga");
            busConfiguration.UseTransport<AzureStorageQueueTransport>()
                .ConnectionString(connectionString);
            busConfiguration.UsePersistence<AzureStoragePersistence, StorageType.Sagas>()
                .ConnectionString(connectionString);
            busConfiguration.UsePersistence<AzureStoragePersistence, StorageType.Timeouts>()
                .ConnectionString(connectionString);
            busConfiguration.UseSerialization<JsonSerializer>();
            busConfiguration.EnableInstallers();

            var random = new Random();

            using (var bus = Bus.Create(busConfiguration).Start())
            {
                Console.WriteLine("Press 'enter' to send a message");
                Console.WriteLine("Press any other key to exit");

                while (true)
                {
                    var key = Console.ReadKey();
                    Console.WriteLine();

                    if (key.Key != ConsoleKey.Enter)
                    {
                        return;
                    }

                    var inventoryId = random.Next(1000, 10000).ToString();
                    var message = new StartImportInventoryImagesMessage
                    {
                        InventoryId = inventoryId,
                        Description = $"kicking off saga for inventory id={inventoryId}"
                    };
                    bus.Send(message);
                    Console.WriteLine($"Message with InventoryId={message.InventoryId} was sent");
                }
            }
        }
    }

    class ConfigureErrorQueue : IProvideConfiguration<MessageForwardingInCaseOfFaultConfig>
    {
        public MessageForwardingInCaseOfFaultConfig GetConfiguration()
        {
            return new MessageForwardingInCaseOfFaultConfig
            {
                ErrorQueue = "error"
            };
        }
    }

    class ConfigureMessageMapping : IProvideConfiguration<UnicastBusConfig>
    {
        public UnicastBusConfig GetConfiguration()
        {
            return new UnicastBusConfig
            {
                MessageEndpointMappings = new MessageEndpointMappingCollection
                {
                    new MessageEndpointMapping
                    {
                        Namespace = "Messages",
                        Endpoint = "EndpointWithSaga",
                        AssemblyName = Assembly.GetExecutingAssembly().GetName().Name
                    }
                }
            };
        }
    }
}
